==> s-create-react-app-1.0.0.tgz: package/createRectApp.js <==
import chalk from 'chalk';
import fs from 'fs-extra'
import path from 'path'
import os from 'os'
import semver from 'semver';
import { Command } from 'commander'
import { execSync, spawn } from 'child_process';

import packageJson from './package.json' assert {type: "json"};


let projectName;

function isUsingYarn() {
  return (process.env.npm_config_user_agent || '').indexOf('yarn') === 0;
}
function checkNpVersion() {
  let hasMinNpm = false;
  let npmVersion = null;
  try {
    npmVersion = execSync('npm --version').toString().trim()
    hasMinNpm = semver.gte(npmVersion, '6.0.0')
  } catch (err) {
    console.log(err.message);
  }
  return {
    hasMinNpm,
    npmVersion
  }
}
function checkIfOnline(useYarn) {
  if (!useYarn) {
    return Promise.resolve(true)
  }
}

function init() {
  const program = new Command(packageJson.name)
    .version(packageJson.version)
    .argument("<proejct-directory>")
    .usage(`${chalk.green('<project-directory>')} [options]`)
    .action(name => {
      console.log(name, 1111);
      projectName = name
    })
    .option('--info', "print info logs")
    .parse(process.argv);
  if (program.info) {
    console.log('input the info');
  }

  createApp({
    projectName,
    version: program.scriptsVersion
  })
}
function createApp({
  projectName,
  version
}) {
  const root = path.resolve(projectName)
  const appName = path.basename(root)
  fs.ensureDirSync(projectName)
  console.log(`${chalk.green('Creating a new React app in')}${chalk.blueBright(root)}`);

  const packageJson = {
    name: appName,
    version: '1.0.0',
    private: true,
    author: "",
  }
  fs.writeFileSync(
    path.join(root, 'package.json'),
    JSON.stringify(packageJson, null, 2) + os.EOL
  )

  const originalDirectory = process.cwd(); // Node进程的当前工作目录
  process.chdir(root) // 更改Node进程的当前目录
  if (!isUsingYarn()) {
    const npmVersion = checkNpVersion()
    if (!npmVersion.hasMinNpm) {
      if (npmInfo.npmVersion) {
        console.log(`
          ${chalk.yellow('your npm version is too old, please update to npm 6 or higher!')}
        `);
      }
    }
  }

  run({
    root,
    appName,
    originalDirectory,
  })
}
function getInstallPackage(_, originalDirectory) {
  let packageToInstall = "react-scripts";

  return Promise.resolve(packageToInstall)
}
function getTemplateInstallPackage(_, originalDirectory) {
  let templateToInstall = 'cra-template';

  return Promise.resolve(templateToInstall)
}
function getPackageInfo(installPakcage) {

  return Promise.resolve({
    name: installPakcage
  })
}

function install({
  root,
  allDepndencies,
  isOnline,
  useYarn = false
}) {

  return new Promise((resolve, reject) => {

    let command;
    let args;

    if (useYarn) {

    } else {
      command = "npm"
      args = [
        "install",
        "--save",
        "--save-exact",
        "--loglevel",
        "error"
      ].concat(allDepndencies)

      const child = spawn(command, args, {
        stdio: 'inherit'
      })
      child.on('close', code => {
        if (code !== 0) {
          return reject({
            command: `${command} ${args.join(' ')}`
          })
        }
        resolve()
      })
    }
  })
}

function executeNodeScript({
  cwd, args
}, data, source) {
  return new Promise((resolve, reject) => {

    const child = spawn(
      process.execPath,
      [...args, '-e', source, '--', JSON.stringify(data)],
      {
        cwd, stdio: 'inherit'
      }
    )
    child.on('close', code => {
      if (code !== 0) {
        return reject({
          command: `node ${args.join(' ')}`
        })
      }
      resolve()
    })
  })
}

function run({
  root,
  appName,
  originalDirectory,
}) {
  Promise.all([
    getInstallPackage(undefined, originalDirectory),
    getTemplateInstallPackage(undefined, originalDirectory)
  ])
    .then(([packageToInstall, templateToInstall]) => {
      const allDepndencies = ["react", 'react-dom', packageToInstall]

      console.log(chalk.green(`Installing packages, this might take a couple of minutes...`));
      Promise.all([
        getPackageInfo(packageToInstall),
        getPackageInfo(templateToInstall),
      ])
        .then(([packageInfo, templateInfo]) => {
          return checkIfOnline(false).then(isOnline => ({
            isOnline,
            packageInfo,
            templateInfo,
          }))
        })
        .then(({
          isOnline,
          packageInfo,
          templateInfo,
        }) => {
          let packageVersion = semver.coerce(packageInfo.version)
          const templatesVersionMinimum = '3.3.0'

          if (!semver.valid(packageVersion)) {
            packageVersion = templatesVersionMinimum
          }

          const supportsTemplates = semver.gte(
            packageVersion,
            templatesVersionMinimum
          )
          if (supportsTemplates) {
            allDepndencies.push(templateToInstall)
          }

          console.log(`Installing ${chalk.cyan('react')}, ${chalk.cyan('react-dom')}, and ${chalk.cyan(packageInfo.name)} ${supportsTemplates ? 'with ' + chalk.cyan(templateInfo.name) : ''
            }
        `);
          return install({
            root,
            allDepndencies,
            isOnline,
          })
            .then(() => ({
              packageInfo,
              supportsTemplates,
              templateInfo
            }))
        })
        .then(({
          packageInfo,
          supportsTemplates,
          templateInfo
        }) => {
          const packageName = packageInfo.name
          const templateName = supportsTemplates ? templateInfo.name : undefined
          // TODO check npm version is safe?
          // TODO set caret ranghe for runtime deps
          executeNodeScript({
            cwd: process.cwd(),
            args: []
          },
          [root, appName, undefined, originalDirectory, templateName],
          `
            const init = require('/Users/jinpeng/Desktop/github/create-react-app/packages/react-scripts/scripts/init.js');
            init.apply(null, JSON.parse(process.argv[1]))
          `
          //   `
          //   const init = require('/Users/jinpeng/Desktop/github/self_create_react_app/packages/s-react-scripts/scripts/init.js');
          //   init.apply(null, JSON.parse(process.argv[1]))
          // `
          )
        }).then(() => {
          console.log("finish create-react-app");
        })
    })
}

export {
  init
}

==> s-create-react-app-1.0.1.tgz: package/createRectApp.js <==
import chalk from 'chalk';
import fs from 'fs-extra'
import path from 'path'
import os from 'os'
import semver from 'semver';
import { Command } from 'commander'
import { execSync, spawn } from 'child_process';

import packageJson from './package.json' assert {type: "json"};


let projectName;

function isUsingYarn() {
  return (process.env.npm_config_user_agent || '').indexOf('yarn') === 0;
}
function checkNpVersion() {
  let hasMinNpm = false;
  let npmVersion = null;
  try {
    npmVersion = execSync('npm --version').toString().trim()
    hasMinNpm = semver.gte(npmVersion, '6.0.0')
  } catch (err) {
    console.log(err.message);
  }
  return {
    hasMinNpm,
    npmVersion
  }
}
function checkIfOnline(useYarn) {
  if (!useYarn) {
    return Promise.resolve(true)
  }
}

function init() {
  const program = new Command(packageJson.name)
    .version(packageJson.version)
    .argument("<proejct-directory>")
    .usage(`${chalk.green('<project-directory>')} [options]`)
    .action(name => {
      console.log(name, 1111);
      projectName = name
    })
    .option('--info', "print info logs")
    .parse(process.argv);
  if (program.info) {
    console.log('input the info');
  }

  createApp({
    projectName,
    version: program.scriptsVersion
  })
}
function createApp({
  projectName,
  version
}) {
  const root = path.resolve(projectName)
  const appName = path.basename(root)
  fs.ensureDirSync(projectName)
  console.log(`${chalk.green('Creating a new React app in')}${chalk.blueBright(root)}`);

  const packageJson = {
    name: appName,
    version: '1.0.0',
    private: true,
    author: "",
  }
  fs.writeFileSync(
    path.join(root, 'package.json'),
    JSON.stringify(packageJson, null, 2) + os.EOL
  )

  const originalDirectory = process.cwd(); // Node进程的当前工作目录
  process.chdir(root) // 更改Node进程的当前目录
  if (!isUsingYarn()) {
    const npmVersion = checkNpVersion()
    if (!npmVersion.hasMinNpm) {
      if (npmInfo.npmVersion) {
        console.log(`
          ${chalk.yellow('your npm version is too old, please update to npm 6 or higher!')}
        `);
      }
    }
  }

  run({
    root,
    appName,
    originalDirectory,
  })
}
function getInstallPackage(_, originalDirectory) {
  let packageToInstall = "s-react-scripts";

  return Promise.resolve(packageToInstall)
}
function getTemplateInstallPackage(_, originalDirectory) {
  let templateToInstall = 's-cra-template';

  return Promise.resolve(templateToInstall)
}
function getPackageInfo(installPakcage) {

  return Promise.resolve({
    name: installPakcage
  })
}

function install({
  root,
  allDepndencies,
  isOnline,
  useYarn = false
}) {

  return new Promise((resolve, reject) => {

    let command;
    let args;

    if (useYarn) {

    } else {
      command = "npm"
      args = [
        "install",
        "--save",
        "--save-exact",
        "--loglevel",
        "error"
      ].concat(allDepndencies)

      const child = spawn(command, args, {
        stdio: 'inherit'
      })
      child.on('close', code => {
        if (code !== 0) {
          return reject({
            command: `${command} ${args.join(' ')}`
          })
        }
        resolve()
      })
    }
  })
}

function executeNodeScript({
  cwd, args
}, data, source) {
  return new Promise((resolve, reject) => {

    const child = spawn(
      process.execPath,
      [...args, '-e', source, '--', JSON.stringify(data)],
      {
        cwd, stdio: 'inherit'
      }
    )
    child.on('close', code => {
      if (code !== 0) {
        return reject({
          command: `node ${args.join(' ')}`
        })
      }
      resolve()
    })
  })
}

function run({
  root,
  appName,
  originalDirectory,
}) {
  Promise.all([
    getInstallPackage(undefined, originalDirectory),
    getTemplateInstallPackage(undefined, originalDirectory)
  ])
    .then(([packageToInstall, templateToInstall]) => {
      const allDepndencies = ["react", 'react-dom', packageToInstall]

      console.log(chalk.green(`Installing packages, this might take a couple of minutes...`));
      Promise.all([
        getPackageInfo(packageToInstall),
        getPackageInfo(templateToInstall),
      ])
        .then(([packageInfo, templateInfo]) => {
          return checkIfOnline(false).then(isOnline => ({
            isOnline,
            packageInfo,
            templateInfo,
          }))
        })
        .then(({
          isOnline,
          packageInfo,
          templateInfo,
        }) => {
          let packageVersion = semver.coerce(packageInfo.version)
          const templatesVersionMinimum = '3.3.0'

          if (!semver.valid(packageVersion)) {
            packageVersion = templatesVersionMinimum
          }

          const supportsTemplates = semver.gte(
            packageVersion,
            templatesVersionMinimum
          )
          if (supportsTemplates) {
            allDepndencies.push(templateToInstall)
          }

          console.log(`Installing ${chalk.cyan('react')}, ${chalk.cyan('react-dom')}, and ${chalk.cyan(packageInfo.name)} ${supportsTemplates ? 'with ' + chalk.cyan(templateInfo.name) : ''
            }
        `);
          return install({
            root,
            allDepndencies,
            isOnline,
          })
            .then(() => ({
              packageInfo,
              supportsTemplates,
              templateInfo
            }))
        })
        .then(({
          packageInfo,
          supportsTemplates,
          templateInfo
        }) => {
          const packageName = packageInfo.name
          const templateName = supportsTemplates ? templateInfo.name : undefined
          // TODO check npm version is safe?
          // TODO set caret ranghe for runtime deps
          executeNodeScript({
            cwd: process.cwd(),
            args: []
          },
          [root, appName, undefined, originalDirectory, templateName],
          // `
          //   const init = require('/Users/jinpeng/Desktop/github/create-react-app/packages/react-scripts/scripts/init.js');
          //   init.apply(null, JSON.parse(process.argv[1]))
          // `
            `
            const init = require('/Users/jinpeng/Desktop/github/self_create_react_app/packages/s-react-scripts/scripts/init.cjs');
            init.apply(null, JSON.parse(process.argv[1]))
          `
          )
        }).then(() => {
          console.log("finish create-react-app");
        })
    })
}

export {
  init
}
